--------------------------------------------------------------------
-- Support functions
--------------------------------------------------------------------


---------------------------
-- format memory usage
---------------------------
function Glance.Functions.FormatMemory(usage)
	if usage > 1000 then
		return format("|cffFF8080%.2f mb", usage/1024)
	elseif usage > 500 then
		return format("|cffffff00%.2f kb", usage)
	elseif usage > 0 then
		return format("|cff00ff00%.2f kb", usage)
	end
end

---------------------------
-- durability % function
---------------------------
function Glance.Functions.GetDurability()
	local have, most, pct = 0,0,0
	local sOut
	for i = 1, 19 do
		local current, max = GetInventoryItemDurability(i)
		if current ~= nil then
			have = have + current
			most = most + max
		end
	end
	pct = math.floor((have/most) * 100)
	if pct >= 75 then
		sOut = "|cff00ff00"..pct
	elseif pct < 75 and pct > 50 then
		sOut = "|cfffbe550"..pct
	else
		sOut = "|cffff0000"..pct
	end
	return sOut
end

---------------------------
-- Clock Min function
---------------------------
function Glance.Functions.GetMin(min)
	if min < 10 then
		return "0"..min
	end
	return min
end

---------------------------
-- format time
---------------------------
function Glance.Functions.formatTime(s)
	local days = 0
	local hours = 0
	local minutes = 0
	local seconds = 0
	local formatted = ""
	if not s or (s < 0) then
		seconds = 0
	else
		days = floor(s/24/60/60); s = mod(s, 24*60*60);
		hours = floor(s/60/60); s = mod(s, 60*60);
		minutes = floor(s/60); s = mod(s, 60);
		seconds = s;
	end
	
	if days ~= nil and days ~= 0 then
		formatted = formatted..days.."d, "
	end
	if hours ~= nil and hours ~= 0 then
		formatted = formatted..hours.."h, "
	end
	if minutes ~= nil and minutes ~= 0 then
		formatted = formatted..minutes.."m, "
	end
	if seconds ~= nil and seconds ~= 0 then
		formatted = formatted..seconds.."s"
	end
	
	return formatted
end

---------------------------
-- Clock Hour function
---------------------------
function Glance.Functions.GetHour(hour)
	if hour > 12 then
		hour = hour - 12
	end
	if hour < 10 then
		return "0"..hour
	end
	return hour
end

---------------------------
-- Clock DST function
---------------------------
function Glance.Functions.GetDST(hour)
	if hour < 12 then
		return "AM"
	else
		return "PM"
	end
end

-- lag format function
---------------------------
function Glance.Functions.formatBandwidth(input)
	local kb = format("%.2f KB/s", input)
	return kb
end

---------------------------
-- lag format function
---------------------------
function Glance.Functions.formatLag(input)
	local lag = input
	local sLag
	if lag >= 350 then
		sLag = "|cffff0000"..lag
	elseif lag < 350 and lag > 200 then
		sLag = "|cfffbe550"..lag
	else
		sLag = "|cff00ff00"..lag
	end
	return sLag
end

---------------------------
-- mem format function
---------------------------
function Glance.Functions.formatMem()
	local total = 0
	UpdateAddOnMemoryUsage()
	for i=1, GetNumAddOns() do
		if IsAddOnLoaded(i) then
			local memory = GetAddOnMemoryUsage(i)
			total = total + memory
		end
	end
	return Glance.Functions.FormatMemory(total)
end

---------------------------
-- fps format function
---------------------------
function Glance.Functions.formatFPS(fps)
	fps = math.floor(fps)
	local sfps
	if fps <= 8 then
		sfps = "|cffff0000"..fps
	elseif fps > 8 and fps < 15 then
		sfps = "|cfffbe550"..fps
	else
		sfps = "|cff00ff00"..fps
	end
	return sfps
end

---------------------------
-- color codes
---------------------------
function Glance.Functions.DecimalToHex(r,g,b)
    return string.format("|cff%02x%02x%02x", r*255, g*255, b*255)
end

---------------------------
-- return pct
---------------------------
function Glance.Functions.getPCT(small,large)
	return math.floor(((small/large)*100))
end

---------------------------
-- text matching
---------------------------
function Glance.Functions.match(u,t,find)
	for i = 1, u do
		local text = string.lower(_G[t..i]:GetText())
		for word in string.gmatch(text, "%S+") do
			if strfind(string.lower(word), string.lower(find)) then return true end
		end
	end
	return false
end

---------------------------
-- send message
---------------------------
function Glance.Functions.sendMSG(text)
	print("|cffFF1E00Glance:|r "..tostring(text));
end

---------------------------
-- regex chat log
---------------------------
function Glance.Functions.parseChatLog(isQuest)
	local inInstance, itype = IsInInstance()
	local inGroup =  GetNumPartyMembers() > 0
	local regex = nil
	local isRested = GetXPExhaustion()
	if not isQuest then
		if inInstance and isRested then
			if itype == "party" and inGroup then
				regex = COMBATLOG_XPGAIN_EXHAUSTION1_GROUP
			elseif itype == "raid" and inGroup then
				regex = COMBATLOG_XPGAIN_EXHAUSTION1_RAID
			else
				regex = COMBATLOG_XPGAIN_EXHAUSTION1
			end
		elseif inInstance and not isRested then
			if itype == "party" and inGroup then
				regex = COMBATLOG_XPGAIN_FIRSTPERSON_GROUP
			elseif itype == "raid" and inGroup then
				regex = COMBATLOG_XPGAIN_FIRSTPERSON_RAID
			else
				regex = COMBATLOG_XPGAIN_FIRSTPERSON
			end
		else
			if isRested then
				regex = COMBATLOG_XPGAIN_EXHAUSTION1
			else
				regex = COMBATLOG_XPGAIN_FIRSTPERSON
			end
		end
	else
		regex = COMBATLOG_XPGAIN_FIRSTPERSON_UNNAMED
	end
	regex = string.gsub(regex, "%(", "%%(")
	regex = string.gsub(regex, "%)", "%%)")
	regex = string.gsub(regex, "%+", "%%+")
	regex = string.gsub(regex, "%-", "%%-")
	regex = string.gsub(regex, "%%%d?%$?s", "(.+)")
	regex = string.gsub(regex, "%%%d?%$?d", "(%%d+)")
	return regex
end

---------------------------
-- parse chat log
---------------------------
function Glance.Functions.parseChatMessage(message)
	local isQuest = string.find(message, ",") == nil
	local pattern = Glance.Functions.parseChatLog(isQuest)
	local mob, xp = strmatch(message, pattern);
	if tonumber(mob) then
		xp = tonumber(mob)
		mob = nil
	end	
	return xp, mob;
end

---------------------------
-- on chat xp event
---------------------------
function Glance.Functions.onChatXPGain(message)
    local xp, mobName = Glance.Functions.parseChatMessage(message)
    xp = tonumber(xp)
	if not xp then
		return false
	end	
    if mobName ~= nil then
		Glance.Functions.stats(Glance.Variables.xp.kills,xp)
    else
		if Glance.Variables.questCompleteOpen then
			Glance.Functions.stats(Glance.Variables.xp.quests,xp)
			Glance.Variables.questCompleteOpen = false
		else
			Glance.Functions.stats(Glance.Variables.xp.gathers,xp)
		end
	end
end

---------------------------
-- set xp vars
---------------------------
function Glance.Functions.setXP()
	local gti = Glance.Variables.gametime
	gti.session = time() - gti.initSession
	gti.total = gti.initTotal + gti.session
	gti.level = gti.initLevel + gti.session
	
	local var = Glance.Variables.xp
	var.level         = UnitLevel("player")
	var.cap           = UnitXPMax("player")
	var.current       = UnitXP("player")
	var.currentPCT    = Glance.Functions.getPCT(var.current,var.cap)
	var.needed        = var.cap - var.current
	var.neededPCT     = 100-var.currentPCT
	var.rested        = GetXPExhaustion() or 0
	var.restedPCT     = Glance.Functions.getPCT(var.rested,var.cap)
	var.session       = var.current - var.init + var.accumulated
	var.sessionPCT    = Glance.Functions.getPCT(var.session,var.cap)
	var.xpersec       = (var.session/gti.session) or 0
	var.xperhour      = math.ceil(var.xpersec*3600) or 0
	if var.xpersec == 0 then
		var.leveltime = 0
	else
		var.leveltime = math.ceil(var.needed/var.xpersec)
	end
	if var.kills.avg ~= 0 then
		var.kills2level = math.ceil(var.needed/var.kills.avg) 
		Glance_Options.xp.kills2level = var.kills2level
	else 
		var.kills2level = Glance_Options.xp.kills2level or 0
	end
	if var.quests.avg ~= 0 then
		var.quests2level  = math.ceil(var.needed/var.quests.avg)
		Glance_Options.xp.quests2level = var.quests2level
	else
		var.quests2level = Glance_Options.xp.quests2level or 0
	end
	if var.gathers.avg ~= 0 then
		var.gathers2level = math.ceil(var.needed/var.gathers.avg)
		Glance_Options.xp.gathers2level = var.gathers2level
	else
		var.gathers2level = Glance_Options.xp.gathers2level or 0
	end
	var.petlevel      = UnitLevel("pet")
	var.petcap        = select(2,GetPetExperience()) or 0
	var.petcurrent    = select(1,GetPetExperience()) or 0
	var.petcurrentPCT = Glance.Functions.getPCT(var.petcurrent,var.petcap)
	var.petneeded     = var.petcap-var.petcurrent or 0
	var.petneededPCT  = Glance.Functions.getPCT(var.petneeded,var.petcap)
	Glance_Options.xp.kills2level   = var.kills2level
	Glance_Options.xp.quests2level  = var.quests2level
	Glance_Options.xp.gathers2level = var.gathers2level
end

---------------------------
-- try to send a message
---------------------------
function Glance.Functions.addonQuery(prefix,message)
	if (GetNumPartyMembers() ~= 0 or Glance.Variables.Debug) and Glance_Options.sendStats then
		if Glance.Variables.Debug then
			SendAddonMessage(prefix, message, "WHISPER", UnitName("player"))
		else
			SendAddonMessage(prefix, message, "PARTY")
		end
		if Glance.Variables.Debug then Glance.Functions.sendMSG("DBG: |cFF69CCF0SENDQUERY: "..message) end
	end
end

---------------------------
-- try to receive a msg
---------------------------
function Glance.Functions.addonResponse(prefix, message, channel, sender)
	if ((GetNumPartyMembers() ~= 0 and sender ~= UnitName("player")) or Glance.Variables.Debug) and Glance_Options.sendStats then
		if prefix == "GlanceQuery" then
			local xp,lag,fps = Glance.Variables.xp, select(3,GetNetStats()), Glance.Variables.fps.avg
			local msg = UnitLevel("player").."|"..xp.xperhour.."|"..xp.currentPCT.."|"..lag.."|"..fps..Glance.Functions.GetDurability()
			if Glance.Variables.Debug then
				SendAddonMessage("GlanceResponse",msg, "WHISPER", UnitName("player"))
			else
				SendAddonMessage("GlanceResponse",msg, "PARTY")
			end
			if Glance.Variables.Debug then Glance.Functions.sendMSG("DBG: |cFF69CCF0RESPTO: "..sender..": "..msg) end
		elseif prefix == "GlanceResponse" then
			local writevar
			if Glance.Variables.Debug then
				if sender == UnitName("player") then writevar = Glance.Variables.party.a end
			else
				if sender == UnitName("party1") then writevar = Glance.Variables.party.a end
			end
			if sender == UnitName("party2") then writevar = Glance.Variables.party.b end
			if sender == UnitName("party3") then writevar = Glance.Variables.party.c end
			if sender == UnitName("party4") then writevar = Glance.Variables.party.d end
			writevar.level, writevar.xperhour, writevar.xpercent, writevar.lag, writevar.fps, writevar.armor = strsplit("|", message)
			writevar.name = sender
			if Glance.Variables.Debug then Glance.Functions.sendMSG("DBG: |cFF69CCF0RECVFROM: "..sender..": "..message) end
		end
	end
end

---------------------------
-- min, max, avg
---------------------------
function Glance.Functions.stats(var,num)
	var.current = num
	var.count = var.count + 1
	if (var.count == 1) then
		var.min = var.current
		var.max = var.current
		var.avg = var.current
	else
		if (var.current < var.min) then
			var.min = var.current
		elseif (var.current > var.max) then
			var.max = var.current
		end
		var.avg = math.floor((var.avg * (var.count-1) + var.current) / var.count)
	end
end

---------------------------
-- repair costs
---------------------------
function Glance.Functions.checkInventory()
	CreateFrame("GameTooltip","Glance_Tooltip_Armor",UIParent,"GameTooltipTemplate")
	local TotalRepairCost = 0
	for i=1, 11 do
		local slotId, textureName = GetInventorySlotInfo(Glance.Arrays.slotItems[i])
		local hasItem, hasCooldown, repairCost = Glance_Tooltip_Armor:SetInventoryItem("player", slotId);
		if hasItem then
			TotalRepairCost = TotalRepairCost + (repairCost or 0)
		end
	end
	Glance_Tooltip_Armor:Hide()
	return TotalRepairCost
end

---------------------------
-- store addons
---------------------------
function Glance.Functions.getAddons()
	wipe(Glance.Arrays.addons)
	local i = 0
	UpdateAddOnMemoryUsage()
	for i=1, GetNumAddOns() do
		if IsAddOnLoaded(i) then
			local memory = GetAddOnMemoryUsage(i)
			--if memory ~= nil and memory > 0 and GetAddOnInfo(i) ~= nil then
				table.insert(Glance.Arrays.addons, {GetAddOnInfo(i), memory,"LBL","steady"})
			--end
		end
	end
	table.sort(Glance.Arrays.addons, function(a, b) return a[2] > b[2] end)
end

---------------------------
-- clear info
---------------------------
function Glance.Functions.resetInfo()
	wipe(Glance.Arrays.info)
	Glance.Arrays.info = {
		["disabled"]     = nil,
		["isTitle"]      = nil,
		["hasArrow"]     = nil,
		["notCheckable"] = nil,
		["checked"]      = nil,
		["text"]         = nil,
		["value"]        = nil,
		["fontObject"]   = "GameFontHighlight"
	}
end

---------------------------
-- get rep %
---------------------------
function Glance.Functions.getRep(bv,ev,tv)
	local pct
	if tv > 0 then
		pct = math.floor(((ev-bv)/(tv-bv))*100)
	else
		pct = -(math.floor(((ev-tv)/(tv-bv))*100))
	end
	return pct.."%"
end

---------------------------
-- garbage collection
---------------------------
function Glance.Functions.collectGarbage()
	local current = collectgarbage("count")
	collectgarbage("collect")
	UpdateAddOnMemoryUsage()
	local garbage = current - collectgarbage("count")
	Glance.Functions.sendMSG("Garbage Collected. ("..Glance.Functions.FormatMemory(garbage)..")");
end

---------------------------
-- armor repair
---------------------------
function Glance.Functions.repairArmor()
	if CanMerchantRepair()==1 and Glance_Options.autoRepair then
		local repairAllCost, canRepair = GetRepairAllCost();
		if repairAllCost>0 then
			if canRepair==1 then
				RepairAllItems();
				Glance.Functions.sendMSG("Your items have been repaired for "..GetCoinTextureString(repairAllCost,0))
			else
				Glance.Functions.sendMSG("You don't have enough money for repair!");
			end
		end
	end
	if Glance.Variables.Debug then Glance.Functions.sendMSG("DBG: |cFF69CCF0UDT: Glance.Functions.repairArmor()") end
end

---------------------------
-- get mount info
---------------------------
function Glance.Functions.mountInfo(id)
	GameTooltip_SetDefaultAnchor(Glance_Tooltip_Mount, UIParent)
	local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", id);
	local special, air, fast = false, false, false
	Glance_Tooltip_Mount:SetHyperlink("spell:"..creatureSpellID)
	air = Glance.Functions.airMounts()
	special = Glance.Functions.specialMounts()
	water = Glance.Functions.waterMounts()
	fast = special or Glance.Functions.fastMounts() 
	Glance_Tooltip_Mount:Hide();
	return creatureName, icon, issummoned, air, special, fast, water
end

---------------------------
-- search for flying mounts
---------------------------
function Glance.Functions.airMounts()
	local air = false
	for i=1, #Glance.Arrays.flightMounts do
		air = Glance.Functions.match(Glance_Tooltip_Mount:NumLines(),"Glance_Tooltip_MountTextLeft",Glance.Arrays.flightMounts[i])
		if air then return true end
	end
	return false
end

---------------------------
-- search for special mounts
---------------------------
function Glance.Functions.specialMounts()
	local special = false
	for i=1, #Glance.Arrays.specialMounts do
		special = Glance.Functions.match(Glance_Tooltip_Mount:NumLines(),"Glance_Tooltip_MountTextLeft",Glance.Arrays.specialMounts[i])
		if special then return true end
	end
	return false
end

---------------------------
-- search for fastmounts
---------------------------
function Glance.Functions.fastMounts()
	local fast = false
	for i=1, #Glance.Arrays.fastMounts do
		fast = Glance.Functions.match(Glance_Tooltip_Mount:NumLines(),"Glance_Tooltip_MountTextLeft",Glance.Arrays.fastMounts[i])
		if fast then return true end
	end
	return false
end

---------------------------
-- search for water mounts
---------------------------
function Glance.Functions.waterMounts()
	local water = false
	for i=1, #Glance.Arrays.waterMounts do
		water = Glance.Functions.match(Glance_Tooltip_Mount:NumLines(),"Glance_Tooltip_MountTextLeft",Glance.Arrays.waterMounts[i])
		if water then return true end
	end
	return false
end

---------------------------
-- store mounts in {}
---------------------------
function Glance.Functions.resetMountList()
	wipe(Glance.Arrays.mount)
	local mountListChanged = false
	for i=1,GetNumCompanions("MOUNT") do
		Glance.Arrays.mount[i] = { Glance.Functions.mountInfo(i) }
		if Glance_Options.mounts == nil then Glance_Options.mounts = {} end
		if Glance_Options.mounts[i] == nil then
			if Glance.Arrays.mount[i][6] then
				Glance_Options.mounts[i] = { Glance.Arrays.mount[i][1], "|cff9135B3" ,true }
			else
				Glance_Options.mounts[i] = { Glance.Arrays.mount[i][1], "|cff0070C0" ,true }
			end
			mountListChanged = true
		elseif Glance_Options.mounts[i][1] ~= Glance.Arrays.mount[i][1] then
			if Glance.Arrays.mount[i][6] then
				Glance_Options.mounts[i] = { Glance.Arrays.mount[i][1], "|cff9135B3" ,true }
			else
				Glance_Options.mounts[i] = { Glance.Arrays.mount[i][1], "|cff0070C0" ,true }
			end
			mountListChanged = true
		end
	end
	if mountListChanged then Glance.Functions.sendMSG("Your mount list has changed.  Type '/glance options' to reconfigure your preferences.") end
	if Glance.Variables.Debug then Glance.Functions.sendMSG("DBG: |cFFf40bcbUDT: Glance.Functions.resetMountList()") end
end

---------------------------
-- battleground status
---------------------------
function Glance.Functions.inBattleground()
	local pvpID, localizedName, isActive, canQueue, sec, canEnter = GetWorldPVPAreaInfo(1)
	if (isActive and GetZoneText()=="Wintergrasp") then
		return true;
	end
	pvpID, localizedName, isActive, canQueue, sec, canEnter = GetWorldPVPAreaInfo(2)
	if (isActive and GetZoneText()=="Tol Barad") then
		return true;
	end
	return false;
end

---------------------------
-- summon a random mount
---------------------------
function Glance.Functions.summonRandomMount()
	if IsOutdoors() then
		if IsSwimming() then
			if strfind(GetMapInfo(),"Vashj") then
				Glance.Functions.summonRandomWaterMount()
			elseif strfind(GetMapInfo(),"Thousand") then
				Glance.Functions.summonRandomWaterMount()
			else 
				Glance.Functions.summonRandomWaterMount()
			end
		else
			if (IsFlyableArea() and not Glance.Functions.inBattleground()) then
				Glance.Functions.summonRandomAirMount()
			else
				Glance.Functions.summonRandomLandMount()
			end
		end
	end
end

---------------------------
-- summon a random air mount
---------------------------
function Glance.Functions.summonRandomAirMount()
	local mountCount = 0
	local tmpmount = {}
	for i=1,#Glance.Arrays.mount do
		local creatureName, icon, issummoned, air, special, fast, water = unpack(Glance.Arrays.mount[i])
		local pvpID, localizedName, isActive, canQueue, sec, canEnter = GetWorldPVPAreaInfo(1)
		Glance.Arrays.mount[i][3] = false
		if (air or special) and Glance_Options.mounts[i][3] then
			mountCount = mountCount + 1
			tmpmount[mountCount] = i
		end
	end
	if #tmpmount > 0 then
		local rdm = tmpmount[random(1,#tmpmount)]
		Glance.Arrays.mount[rdm][3] = true
		CallCompanion("MOUNT",rdm)
		if Glance.Variables.Debug then Glance.Functions.sendMSG("DBG: |cffffff00 Calling air: "..rdm) end
	end
	wipe(tmpmount)
end

---------------------------
-- summon a random land mount
---------------------------
function Glance.Functions.summonRandomLandMount()
	local mountCount = 0
	local tmpmount = {}
	for i=1,#Glance.Arrays.mount do
		local creatureName, icon, issummoned, air, special, fast, water = unpack(Glance.Arrays.mount[i])
		Glance.Arrays.mount[i][3] = false
		if not (air or water) and Glance_Options.mounts[i][3] then
			mountCount = mountCount + 1
			tmpmount[mountCount] = i
		end
	end
	if #tmpmount > 0 then
		local rdm = tmpmount[random(1,#tmpmount)]
		Glance.Arrays.mount[rdm][3] = true
		CallCompanion("MOUNT",rdm)
		if Glance.Variables.Debug then Glance.Functions.sendMSG("DBG: |cffffff00 Calling land: "..rdm) end
	end
	wipe(tmpmount)
end

---------------------------
-- summon a random water mount
---------------------------
function Glance.Functions.summonRandomWaterMount()
	local mountCount = 0
	local tmpmount = {}
	for i=1,#Glance.Arrays.mount do
		local creatureName, icon, issummoned, air, special, fast, water = unpack(Glance.Arrays.mount[i])
		Glance.Arrays.mount[i][3] = false
		if water and Glance_Options.mounts[i][3] then
			mountCount = mountCount + 1
			tmpmount[mountCount] = i
		end
	end
	if #tmpmount > 0 then
		local rdm = tmpmount[random(1,#tmpmount)]
		Glance.Arrays.mount[rdm][3] = true
		CallCompanion("MOUNT",rdm)
		if Glance.Variables.Debug then Glance.Functions.sendMSG("DBG: |cffffff00 Calling water: "..rdm) end
	end
	wipe(tmpmount)
end

---------------------------
-- get pet info
---------------------------
function Glance.Functions.petInfo(id)
	local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("CRITTER", id);
	return creatureName, icon, issummoned
end

---------------------------
-- store pets in {}
---------------------------
function Glance.Functions.resetPetList()
	wipe(Glance.Arrays.pet)
	local petListChanged = false
	for i=1,GetNumCompanions("CRITTER") do
		Glance.Arrays.pet[i] = { Glance.Functions.petInfo(i) }
		if Glance_Options.pets == nil then Glance_Options.pets = {} end
		if Glance_Options.pets[i] == nil then
			Glance_Options.pets[i] = { Glance.Arrays.pet[i][1], true }
			petListChanged = true
		elseif Glance_Options.pets[i][1] ~= Glance.Arrays.pet[i][1] then
			Glance_Options.pets[i] = { Glance.Arrays.pet[i][1], true }
			petListChanged = true
		end
	end
	if petListChanged then Glance.Functions.sendMSG("Your pet list has changed.  Type '/glance options' to reconfigure your preferences.") end
	if Glance.Variables.Debug then Glance.Functions.sendMSG("DBG: |cFFf40bcbUDT: Glance.Functions.resetPetList()") end
end